@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2019-2021 Hans Petter Jansson
+/* Copyright (C) 2019-2022 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that turns images into character art.
  *
@@ -19,7 +19,12 @@
 
 #include "config.h"
 
+#include <stdlib.h>
+#include <string.h>
+#include <glib.h>
+
 #include "chafa.h"
+
 #include "internal/chafa-color-table.h"
 #include "internal/chafa-pca.h"
 
